<?php
namespace Bitfoundry\Etnpay\Model;

use Magento\Quote\Model\Quote;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Sales\Model\Order\Payment;
use Magento\Sales\Model\Order\Payment\Transaction;

class Etnpay extends \Magento\Payment\Model\Method\AbstractMethod
{
    protected $_code = Config::METHOD_CODE;
    protected $_formBlockType = 'Bitfoundry\Etnpay\Block\Form';
    protected $_infoBlockType = 'Bitfoundry\Etnpay\Block\Payment\Info';
    protected $_configType = 'Bitfoundry\Etnpay\Model\Config';
    //protected $_isInitializeNeeded = true;
    protected $_isGateway = false;
    protected $_canOrder = true;
    //protected $_canAuthorize = true;
    protected $_canCapture = true;
    //protected $_canVoid = false;
    //protected $_canUseInternal = true;
    protected $_canUseCheckout = true;
    //protected $_canFetchTransactionInfo = true;
    //protected $_canReviewPayment = true;
    protected $_config;
    //protected $_isOrderPaymentActionKey = 'is_order_action';
    //protected $_authorizationCountKey = 'authorization_count';
    protected $_storeManager;
    protected $_urlBuilder;
    protected $_checkoutSession;
    protected $_exception;
    protected $transactionRepository;
    protected $transactionBuilder;
    protected $etnpayHelper;
    protected $_logger;
    protected $_payment;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param \Bitfoundry\Etnpay\Model\ConfigFactory $configFactory
     * @param \Bitfoundry\Etnpay\Helper\Data $etnpayHelper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\Exception\LocalizedExceptionFactory $exception
     * @param \Magento\Sales\Api\TransactionRepositoryInterface $transactionRepository
     * @param Transaction\BuilderInterface $transactionBuilder
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct( \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        ConfigFactory $configFactory,
        \Bitfoundry\Etnpay\Helper\Data $etnpayHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\Exception\LocalizedExceptionFactory $exception,
        \Magento\Sales\Api\TransactionRepositoryInterface $transactionRepository,
        \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transactionBuilder,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [] )
    {
        parent::__construct( $context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data );

        $this->_storeManager         = $storeManager;
        $this->_urlBuilder           = $urlBuilder;
        $this->_checkoutSession      = $checkoutSession;
        $this->_exception            = $exception;
        $this->transactionRepository = $transactionRepository;
        $this->transactionBuilder    = $transactionBuilder;
        $this->etnpayHelper          = $etnpayHelper;
        $this->_logger               = $logger;

        $parameters = ['params' => [$this->_code]];

        $this->_config = $configFactory->create( $parameters );
    }

    public function setStore( $store )
    {
        $this->setData( 'store', $store );

        if ( null === $store ) {
            $store = $this->_storeManager->getStore()->getId();
        }
        $this->_config->setStoreId( is_object( $store ) ? $store->getId() : $store );

        return $this;
    }

    public function canUseForCurrency( $currencyCode )
    {
        return $this->_config->isCurrencyCodeSupported( $currencyCode );
    }

    public function getConfigPaymentAction()
    {
        return $this->_config->getPaymentAction();
    }

    public function isAvailable( \Magento\Quote\Api\Data\CartInterface $quote = null )
    {
        return parent::isAvailable( $quote ) && $this->_config->isMethodAvailable();
    }

    protected function getStoreName()
    {
        $storeName = $this->_scopeConfig->getValue(
            'general/store_information/name',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        return $storeName;
    }

    /**
     * Get data required to generate Electroneum QR code
     */
    public function getEtnData()
    {
        $helper = $this->etnpayHelper;
        $etnData = array();

        $order = $this->_checkoutSession->getLastRealOrder();

        $etnData['title'] = $this->getConfigData('title');
        $etnData['vendorAddress'] = $this->getConfigData('vendor_address');
        $etnData['amount'] = $this->getTotalAmount($order);
        $etnData['etnAmount'] = $helper->getETNAmount($order->getOrderCurrencyCode(), $this->getTotalAmount($order));
        $etnData['merchantReference'] = $order->getRealOrderId();
        $etnData['shortMerchantName'] = $this->_storeManager->getStore()->getName();
        $etnData['currencyISOCode'] = $order->getOrderCurrencyCode();
        $etnData['etnUrl'] = "etn-it-" . $etnData['vendorAddress'] . "/0" . $etnData['merchantReference'] ."/" . $etnData['etnAmount'];
        $etnData['orderId'] = $order->getId();

        return $etnData;
    }

    public function getVendorAddress() {
        return $this->getConfigData('vendor_address');
    }

    public function getApiKey() {
        return $this->getConfigData('vendor_key');
    }

    public function getApiSecret() {
        return $this->getConfigData('vendor_secret');
    }

    public function getEtnStatus($paymentId)
    {
        $helper = $this->etnpayHelper;

        // get payment confirmation from Electroneum API to confirm order
        $vendorAddress = $this->getVendorAddress();
        $apiKey = $this->getApiKey();
        $apiSecret = $this->getApiSecret();
        $payload = [
            'payment_id' => '0' . $paymentId,
            'vendor_address' => 'etn-it-' . $vendorAddress
        ];

        $etnSignature = $helper->createSecuritySignature($payload, $apiSecret);
        $etnUrl = 'https://poll.electroneum.com/vendor/check-payment';

        $ch = curl_init($etnUrl);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'ETN-SIGNATURE: ' . $etnSignature
        ]);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($payload));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($ch);
        curl_close($ch);

        // Decode the JSON result into an array.
        $result = json_decode($result, true);

        return $result;
    }

    public function getTotalAmount( $order )
    {
        $price = $this->getNumberFormat( $order->getBaseGrandTotal() );
        return $price;
    }

    public function getNumberFormat( $number )
    {
        return number_format( $number, 2, '.', '' );
    }

    public function getPaidSuccessUrl()
    {
        return $this->_urlBuilder->getUrl( 'etnpay/scan/success', array( '_secure' => true ) );
    }

    protected function getOrderTransaction( $payment )
    {
        return $this->transactionRepository->getByTransactionType( Transaction::TYPE_ORDER, $payment->getId(), $payment->getOrder()->getId() );
    }

    public function getOrderPlaceRedirectUrl()
    {
        return $this->_urlBuilder->getUrl( 'etnpay/scan' );

    }

    public function getCheckoutRedirectUrl()
    {
        return $this->_urlBuilder->getUrl( 'etnpay/scan' );
    }

    public function initialize( $paymentAction, $stateObject )
    {
        $stateObject->setState( \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT );
        $stateObject->setStatus( 'pending_payment' );
        $stateObject->setIsNotified( false );

        return parent::initialize( $paymentAction, $stateObject );
    }
}
